SystemOrganization addCategory: #StartupPreferences!

!DosFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenComan 5/8/2012 23:56'!
configFolderLocalName
	^ '..config'! !

!MacFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenComan 5/8/2012 23:56'!
configFolderLocalName
	^ '.config'! !

Object subclass: #PreferencesHandler
	instanceVariableNames: 'firstLink actions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StartupPreferences'!

!PreferencesHandler commentStamp: '<historical>' prior: 0!
I am the one which manage the links of the chain of responsibilites to retreive the good preference file!

!PreferencesHandler methodsFor: 'actions' stamp: 'MarianoMartinezPeck 5/9/2012 19:03'!
buildActionList

	"First, we look at general preferences folder then at the preferences folder, and finally at general preferences folder. All found files will be executed."
	
	| directories |
	directories := OrderedCollection new.
	directories add: [self lookInGeneralPreferencesFolder ].
	directories add: [self lookInPreferencesVersionFolder ].
	directories add: [ self lookInImageDirectory ].
	^ directories! !

!PreferencesHandler methodsFor: 'initialization' stamp: 'MarianoMartinezPeck 5/7/2012 22:06'!
initialize
	"Initialization code for PreferencesHandler"

	super initialize.
	actions := self buildActionList.! !

!PreferencesHandler methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/7/2012 18:19'!
lookInFolder: folder

	| result |

	result := OrderedCollection new.
	folder exists ifFalse: [ ^ nil ].
	
	folder fileNames do: [:name |
		result add: (folder fileNamed: name)].

	^ result isEmpty
		ifTrue: [ nil ]
		ifFalse: [ result ]! !

!PreferencesHandler methodsFor: 'actions' stamp: 'MarianoMartinezPeck 5/7/2012 18:18'!
lookInGeneralPreferencesFolder

	^ self lookInFolder: FileDirectory preferencesGeneralFolder.! !

!PreferencesHandler methodsFor: 'actions' stamp: 'MarianoMartinezPeck 5/7/2012 19:00'!
lookInImageDirectory

	| folder |
	
	folder := FileDirectory default.
	folder exists ifFalse: [ ^ nil ].
	
	^ (folder fileExists: StartupLoader startupScriptName)
			ifTrue:	[ { folder fileNamed: StartupLoader startupScriptName }]
			ifFalse: [ nil ]! !

!PreferencesHandler methodsFor: 'actions' stamp: 'MarianoMartinezPeck 5/9/2012 19:03'!
lookInPreferencesVersionFolder

	^ self lookInFolder: FileDirectory preferencesVersionFolder.
	! !

!PreferencesHandler methodsFor: 'actions' stamp: 'MarianoMartinezPeck 5/7/2012 18:15'!
perform
	| results |
	results := OrderedCollection new.
	actions do: [:each |
		each value ifNotNil: [:res |  results addAll: res ]
	].
	^ results
! !

Object subclass: #StartupAction
	instanceVariableNames: 'code hasBeenExecuted runOnce nameOfItem name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StartupPreferences'!

!StartupAction commentStamp: '<historical>' prior: 0!
I am an atomic item used to execute a portion of code at image startup.!

!StartupAction class methodsFor: 'instance creation' stamp: 'IgorStasenko 2/4/2012 19:09'!
name: nameOfItem code: code

	^ self new
		code: code;
		name: nameOfItem;
		yourself! !

!StartupAction class methodsFor: 'instance creation' stamp: 'IgorStasenko 2/4/2012 19:09'!
name: nameOfItem code: code runOnce: aBoolean

	^ self new
		code: code;
		name: nameOfItem;
		runOnce: aBoolean;
		yourself! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:07'!
= anObject

	anObject class = self class ifFalse: [ ^ false ].
	^ self code = anObject code and: [ self name = anObject name and: [ self runOnce = anObject runOnce ]]! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:06'!
code

	^ code! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:05'!
code: anObject
	
	code := anObject! !

!StartupAction methodsFor: 'execution' stamp: 'MarianoMartinezPeck 5/3/2012 23:11'!
execute

	| block correctlyExecuted |
	(self hasBeenExecuted and: [ self runOnce ]) ifTrue: [ ^ self ].
	block := self code isBlock
				ifTrue: [ self code ]
				ifFalse: [[ self class compilerClass evaluate: self code ]].
	correctlyExecuted := true. 
	block on: Halt,Error do: [:ex | StartupLoader default errors add: {ex. self}. correctlyExecuted := false].
	self hasBeenExecuted: correctlyExecuted.! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:06'!
hasBeenExecuted

	^ hasBeenExecuted ifNil: [ hasBeenExecuted := false ]! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:05'!
hasBeenExecuted: anObject
	
	hasBeenExecuted := anObject! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:07'!
name

	^ name! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:07'!
name: anObject
	
	name := anObject! !

!StartupAction methodsFor: 'printing' stamp: 'IgorStasenko 2/4/2012 19:07'!
printOn: aStream

	aStream <<'a ' << self class name << '(' << self name << ')'! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 18:10'!
runOnce

	^ runOnce ifNil: [ runOnce := false ]! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 18:11'!
runOnce: anObject
	
	runOnce := anObject! !

!StartupAction methodsFor: 'printing' stamp: 'IgorStasenko 2/4/2012 19:07'!
storeOn: aStream

	aStream << self class name << ' name: '<< self name printString <<' code: '<< self code printString.
	self runOnce ifTrue: [ aStream << ' runOnce: true']! !

Object subclass: #StartupLoader
	instanceVariableNames: 'errors actions'
	classVariableNames: 'AllowStartupScript UniqueInstance'
	poolDictionaries: ''
	category: 'StartupPreferences'!

!StartupLoader commentStamp: '<historical>' prior: 0!
I am in charge of the loading of the startup scripts from ~/.config/pharo/#{version}/startup.st or ~/.config/pharo/general/pharo#{version}.st

StartupLoader example

will define a script sample startup.st in your unix root on unix 

Its contents is 

StartupLoader default executeAtomicItems: {
	StartupAtomicItem name: 'Open Help' code: 'Workspace openContents: ''Here is just an example of how to use the StartupLoader.
I should only be displayed once.
	
You can also see StartupLoader class>>#example'' label: ''Help''' isSingleton: true.
	StartupAtomicItem name: 'Open Workspace' code: 'Workspace openContents: ''I should be displayed each time'''.
}

For a more complete example, see StartupLoader class>>#example2!

!StartupLoader class methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:17'!
allowStartupScript

	^ AllowStartupScript ifNil: [ AllowStartupScript := true ]! !

!StartupLoader class methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:17'!
allowStartupScript: aBoolean

	AllowStartupScript := aBoolean! !

!StartupLoader class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/1/2011 08:56'!
default

	^ UniqueInstance ifNil: [ UniqueInstance := super basicNew initialize ]! !

!StartupLoader class methodsFor: 'example' stamp: 'MarianoMartinezPeck 5/6/2012 19:32'!
example
	" an example of script generation "
	| item1 item2 |
	item1 := StartupAction name: 'Open Help' code: 'Workspace openContents: ''Here is just an example of how to use the StartupLoader.
I should only be displayed once.
	
 You can also see StartupLoader class>>#example'' label: ''Help''' runOnce: true.
	item2 := StartupAction name: 'Open Workspace' code:  [ Workspace openContents: 'I should be displayed each time' ].
	StartupLoader default addAtStartupInGeneralPreferenceFolder: {item1. item2}.
	
	StartupLoader default loadStartupScript.
	StartupLoader default loadStartupScript.! !

!StartupLoader class methodsFor: 'example' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
example2
	" it's my personal script provided as example"
	"self example2"
	| items |
	items := OrderedCollection new.
	items add: (StartupAction name: 'Set the Author' code: [ Author fullName: Author fullName printString ]). "replace it by your name"
	
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'author.st'.
	items removeAll.
	
	items add: (StartupAction name: 'Debugger option' code: [ Debugger alwaysOpenFullDebugger: true ]).
	items add: (StartupAction name: 'Dragging Option' code: [ UITheme defaultSettings fastDragging: true ]).
	items add:  (StartupAction name: 'CMD Left Shortcut' code: [ TasklistMorph useHistoricalShortcuts: false ]).
	items add: (StartupAction name: 'Dialog Auto Accept' code: [ TextEditorDialogWindow autoAccept: true ]).
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'settings.st'.
	items removeAll.
	
	items add: (StartupAction name: 'Fonts option' code: [ FreeTypeSystemSettings loadFt2Library: true.
FreeTypeFontProvider current updateFromSystem.
StandardFonts defaultFont: (LogicalFont familyName: 'Lucida Grande' pointSize: 10) forceNotBold.
GraphicFontSettings resetAllFontToDefault.
StandardFonts codeFont: (LogicalFont familyName: 'Consolas' pointSize: 10).] runOnce: true).
	
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'fonts.st'.
	items removeAll.
	StartupLoader default addAtStartupInImageDirectory: items.
	
	StartupLoader default loadStartupScript.! !

!StartupLoader class methodsFor: 'example' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
example3
	" it's my personal script provided as example"
	"self example3"
	| items |
	items := OrderedCollection new.
	items add: (StartupAction name: 'Self halt' code: [ self halt ]). "replace it by your name"
	items add: (StartupAction name: 'WS' code: [ Workspace open ]). "replace it by your name"
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'author.st'.

	StartupLoader default loadStartupScript.! !

!StartupLoader class methodsFor: 'example' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
example4
	"self example4"
	| items |
	items := OrderedCollection new.
	
	items add: (StartupAction name: 'General Preferences for all Pharo versions' code: [ 
		FileStream stdout lf; nextPutAll: 'Setting general preferences for all Pharo versions'; lf.
		FileStream stdout lf; nextPutAll: 'Finished'; lf.
		 ]).
	StartupLoader default addAtStartupInGeneralPreferenceFolder: items named: 'generalSettings.st'.
	
	items removeAll.
	items add: (StartupAction name: 'Settings' code: [ 
		FileStream stdout lf; nextPutAll: 'Setting general preferences for Pharo 2.0'; lf.
		FileStream stdout lf; nextPutAll: 'Finished'; lf.
		 ]).
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'settings.st'.
	
	items removeAll.
	items add: (StartupAction name: 'Image diretory' code: [ 
		FileStream stdout lf; nextPutAll: 'Setting preferences for image directory'; lf.
		FileStream stdout lf; nextPutAll: 'Finished'; lf.
		 ]).
	StartupLoader default addAtStartupInImageDirectory: items ! !

!StartupLoader class methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 11/1/2011 10:39'!
initialize

	super initialize.
	Smalltalk addToStartUpList: self.! !

!StartupLoader class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/1/2011 08:56'!
new

	^ self error: 'use #default instead'! !

!StartupLoader class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/1/2011 09:14'!
reset

	UniqueInstance := nil! !

!StartupLoader class methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 2/10/2012 18:44'!
startUp: resuming

	(self allowStartupScript and: [ resuming ])
		ifTrue: [ Smalltalk addDeferredStartupAction: [ self default loadStartupScript ]]! !

!StartupLoader class methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/7/2012 18:59'!
startupGeneralPreferencesFolderScriptName

	^ 'startupPharo.st'! !

!StartupLoader class methodsFor: 'menu' stamp: 'MarcusDenker 3/24/2012 17:07'!
startupLoaderMenuOn: aBuilder 
	<worldMenu>
	
	(aBuilder item: #startupLoader)
		action: [ self default loadStartupScript ];
		label: 'Run startup scripts';
		parent: #System;
		order: 2;
		help: 'Run startup "scripts';
		icon: UITheme current smallJustifiedIcon! !

!StartupLoader class methodsFor: 'setting' stamp: 'IgorStasenko 2/4/2012 19:17'!
startupPreferencesSettingsOn: aBuilder
	<systemsettings>
	
	(aBuilder group: #'Startup Preferences')
		target: self;
		label: 'Startup Preferences' translated;
		description: 'Settings related to Startup Preferences' translated;
		with: [
			(aBuilder setting: #allowStartupScript)
				label: 'Allow the preferences to be loaded at startup' translated;
				description: 'If true, the preferences will be loaded' translated ]! !

!StartupLoader class methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
startupPreferencesVersionFolderScriptName

	^ 'startupPharo',SystemVersion current dottedMajorMinor,'.st'! !

!StartupLoader class methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/7/2012 18:59'!
startupScriptName

	^ 'startup.st'! !

!StartupLoader methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:06'!
actions

	^ actions! !

!StartupLoader methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:08'!
add: anAction

	^ actions at: anAction name ifAbsentPut: [ anAction ]! !

!StartupLoader methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/6/2012 19:24'!
addAtStartup: aCollection inDirectory: aDirectory named: fileName

	aDirectory assureExistence.
	aDirectory forceNewFileNamed: fileName do: [:stream | stream nextPutAll: (self buildStreamFor: aCollection) ].
! !

!StartupLoader methodsFor: 'script generation' stamp: 'MarianoMartinezPeck 5/7/2012 19:00'!
addAtStartupInGeneralPreferenceFolder: aCollection 
	self addAtStartup: aCollection inDirectory: FileDirectory preferencesGeneralFolder named: self class startupGeneralPreferencesFolderScriptName! !

!StartupLoader methodsFor: 'script generation' stamp: 'MarianoMartinezPeck 5/6/2012 19:31'!
addAtStartupInGeneralPreferenceFolder: aCollection named: fileName
	self addAtStartup: aCollection inDirectory: FileDirectory preferencesGeneralFolder named: fileName! !

!StartupLoader methodsFor: 'script generation' stamp: 'MarianoMartinezPeck 5/7/2012 18:59'!
addAtStartupInImageDirectory:  aCollection
	self addAtStartup: aCollection inDirectory: FileDirectory default named: self class startupScriptName
! !

!StartupLoader methodsFor: 'script generation' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
addAtStartupInPreferenceVersionFolder: aCollection 
	self addAtStartup: aCollection inDirectory: FileDirectory preferencesVersionFolder named: self class startupPreferencesVersionFolderScriptName! !

!StartupLoader methodsFor: 'script generation' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
addAtStartupInPreferenceVersionFolder: aCollection named: fileName
	self addAtStartup: aCollection inDirectory: FileDirectory preferencesVersionFolder named: fileName! !

!StartupLoader methodsFor: 'private' stamp: 'BenjaminVanRyseghem 12/14/2011 13:42'!
buildStreamFor: aCollection
	| stream |
	
	stream := WriteStream on: ''.

	stream
		nextPutAll: 'StartupLoader default executeAtomicItems: {';
		nextPutAll: String crlf.
	aCollection do: [:item |
		stream 
			nextPut: Character tab;
			nextPutAll: item storeString;
			nextPut: $.;
			nextPutAll: String crlf ].
	stream
		nextPut: $};
		nextPut: $.;
		nextPutAll: String crlf;
		nextPutAll: String crlf.
	^ stream contents! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/7/2012 18:14'!
cleanSavedActionsAndErrors
	actions := Dictionary new. 
	errors := OrderedCollection new. ! !

!StartupLoader methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 17:45'!
errors

	^ errors! !

!StartupLoader methodsFor: 'code execution' stamp: 'BenjaminVanRyseghem 3/20/2012 11:18'!
executeAtomicItems: aCollection
	"No sender but send from the generated script"

	'Load Startup Preferences'
		displayProgressFrom: 0 to: aCollection size
		during: [:bar | 
			aCollection doWithIndex: [:item :index | 
				bar value: index.
				(self add: item) execute ]]! !

!StartupLoader methodsFor: 'initialize' stamp: 'IgorStasenko 2/4/2012 19:11'!
initialize

	super initialize.
	actions := Dictionary new. 
	errors := OrderedCollection new. ! !

!StartupLoader methodsFor: 'startup loading' stamp: 'BenjaminVanRyseghem 12/13/2011 11:29'!
loadPreferences

	self retreiveFilesStream do: [:file | file fileIn ].! !

!StartupLoader methodsFor: 'startup loading' stamp: 'BenjaminVanRyseghem 12/13/2011 11:28'!
loadStartupScript

	errors removeAll.
	self loadPreferences.! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/7/2012 18:15'!
removeAllScriptsAndCleanSavedActions
	self removeAllScriptsFromAllDirectories.
	self cleanSavedActionsAndErrors.
	! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/9/2012 19:28'!
removeAllScriptsFromAllDirectories
	self removeScriptFromImageDirectory.
	self removeAllScriptsFromPreferencesVersionDirectory.
	self removeAllScriptsFromGeneralPreferencesDirectory.
! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/7/2012 17:48'!
removeAllScriptsFromDirectory: aDirectory
	aDirectory deleteLocalFiles! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/7/2012 17:46'!
removeAllScriptsFromGeneralPreferencesDirectory
	self removeAllScriptsFromDirectory: FileDirectory preferencesGeneralFolder.
	
	
! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/9/2012 19:28'!
removeAllScriptsFromPreferencesVersionDirectory
	self removeAllScriptsFromDirectory: FileDirectory preferencesVersionFolder.

! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/7/2012 18:59'!
removeScriptFromImageDirectory
	FileDirectory default deleteFileNamed: self class startupScriptName.
! !

!StartupLoader methodsFor: 'startup loading' stamp: 'BenjaminVanRyseghem 12/13/2011 11:29'!
retreiveFilesStream

	^ PreferencesHandler new perform! !

!UnixFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenComan 5/8/2012 23:55'!
configFolderLocalName
	^ '.config'! !

!FileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenComan 5/9/2012 00:24'!
configFolderLocalName
	^ DirectoryClass configFolderLocalName
! !

!FileDirectory class methodsFor: '*StartupPreferences' stamp: 'MarianoMartinezPeck 5/9/2012 19:09'!
preferencesGeneralFolder

	^ self preferencesRootFolder! !

!FileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenComan 5/9/2012 00:22'!
preferencesRootFolder
	
	^ (self default ascendingSearchForDirectory: self configFolderLocalName) directoryNamed: 'pharo'.! !

!FileDirectory class methodsFor: '*StartupPreferences' stamp: 'MarianoMartinezPeck 5/9/2012 19:01'!
preferencesVersionFolder

	^ self preferencesRootFolder directoryNamed: SystemVersion current dottedMajorMinor .
! !

!FileDirectory methodsFor: '*StartupPreferences' stamp: 'BenComan 5/9/2012 00:18'!
ascendingSearchForDirectory: relativeDirectoryName
	"First look for relativeDirectoryName in myself, then in each of my parents proceeding towards root."
	
	(self directoryExists: relativeDirectoryName) ifTrue: [ ^self directoryNamed:  relativeDirectoryName  ].
	self pathName ifEmpty: [ ^self ] .
	^self containingDirectory ascendingSearchForDirectory: relativeDirectoryName.

! !
StartupLoader initialize!
